/* File: TreeRecursion.java
 * Author: Wayne Snyder
 * Date: Aug 1, 2020
 * Purpose: This is the template for HW 5, CS 112; it is simply
 *          a collection of different algorithms for manipulating
 *          binary trees and BSTs. 
 */


public class TreeRecursion {

  // Basic Node definition from class web page

  private static class Node {
    int item;
    Node left;
    Node right;

    public Node(int k) {
      item = k;
      left = null;
      right = null;
    }

    public Node(int n, Node left, Node right) {
      item = n;
      this.left = left;
      this.right = right;
    }
  }





  // Recursively insert into tree

  private static Node insert(Node t, int item) {
    if (t == null)
      return new Node(item);
    else if (item < t.item) {
      t.left = insert(t.left, item);
      return t;
    } else if (item > t.item) {
      t.right = insert(t.right, item);
      return t;
    } else // item is already here, do nothing
      return t;
  }

 

  // Recursively print out a diagram of the tree sideways

  private static void printIndentedTree(Node t) {
    System.out.println();
    printIndentedTreeHelper(t, "");
    System.out.println();
  }

  private static void printIndentedTreeHelper(Node t, String indent) {
    if (t != null) {
      printIndentedTreeHelper(t.right, indent + "     "); // add five spaces to indent
      // System.out.println(indent + t.item + "(" + t.N + ")");
      System.out.println(indent + t.item);
      printIndentedTreeHelper(t.left, indent + "     "); // add five spaces to indent
    }
  }

  // Just a utility method to insert into a tree

  private static Node insertArray(int[] A, Node r) {
    Node t = null;
    for (int i = 0; i < A.length; ++i)
      t = insert(t, A[i]);
    return t;
  }
  
  
  // YOUR CODE HERE

 


  // Unit Test for HW05

  public static void main(String[] args) {

    System.out.println("Sample trees for testing tree methods for HW07:");

    System.out.println("\nTree 0 is null.\n");
    Node root0 = null;

    Node root1 = null;

    int[] A = {5, 2, 9, 4, 1, 7, 12};
    root1 = insertArray(A, root1);
    System.out.println("Tree 1:");
    printIndentedTree(root1);

    Node root2 = null;

    int[] B = {1, 2, 3, 6, 5, 4};
    root2 = insertArray(B, root2);
    System.out.println("Tree 2:");
    printIndentedTree(root2);

    Node root3 = null;

    int[] C = {1, 6, 2, 4, 3, 5};
    root3 = insertArray(C, root3);
    System.out.println("Tree 3:");
    printIndentedTree(root3);

    Node root4 = null;

    int[] D = {6, 2, 15, 9, 4, 1, 18, 3, 12, 7, 5, 10, 13};
    root4 = insertArray(D, root4);
    System.out.println("Tree 4:");
    printIndentedTree(root4);

    System.out.println("\nPerformance Tests....");
 /*   
    System.out.println("\nTesting size....");
    System.out.println("\n[1] Should print out:\n0");
    System.out.println(size(root0));

    System.out.println("\n[2] Should print out:\n7");
    System.out.println(size(root1));
    
    System.out.println("\n[3] Should print out:\n6");
    System.out.println(size(root2));
    
    
    System.out.println("\nTesting treeToString....");
    System.out.println("\n[4] Should print out:\n((( 1 ) 2 ( 4 )) 5 (( 7 ) 9 ( 12 )))");
    System.out.println(treeToString(root1));

    System.out.println("\n[5] Should print out:\n( 1 ( 2 ( 3 ((( 4 ) 5 ) 6 ))))");
    System.out.println(treeToString(root2));

    System.out.println("\nTesting treeToString2....");
    System.out.println("\n[6] Should print out:\n5(2(1(.,.),4(.,.)),9(7(.,.),12(.,.)))");
    System.out.println(treeToString2(root1));

    System.out.println("\n[7] Should print out:\n1(.,6(2(.,4(3(.,.),5(.,.))),.))");
    System.out.println(treeToString2(root3));

    System.out.println("\nTesting numLeaves ....");
    System.out.println("\n[8] Should print out:\n0");
    System.out.println(numLeaves(root0));

    System.out.println("\n[9] Should print out:\n7");
    System.out.println(numLeaves(root4));
    
    System.out.println("\n[10] Should print out:\n1");
    System.out.println(numLeaves(root2));
    
    System.out.println("\nTesting copy ....");   

    System.out.println("\n[11] Should print out:\n(( 1 ) 3 (( 7 ) 10 ))");
    Node temp = insertArray(new int[] {3,1,10}, null);
    Node temp2 = copy(temp);
    temp2.right.item = 13; 
    temp = insert(temp,7);
    System.out.println(treeToString(temp));

    System.out.println("\n[12] Should print out:\n(( 1 ) 3 ( 13 ))");
    System.out.println(treeToString(temp2));
    
    System.out.println("\nTesting reverse ....");  
    
    System.out.println("\n[13] Should print out:\n((( 12 ) 9 ( 7 )) 5 (( 4 ) 2 ( 1 )))");
    System.out.println(treeToString(reverse(copy(root1))));

    System.out.println("\n[14] Should print out:\n(((( 6 ( 5 ( 4 ))) 3 ) 2 ) 1 )");
    System.out.println(treeToString(reverse(copy(root2))));

    System.out.println("\nTesting isBST ...."); 
    
    // not BSTs
    temp = new Node(4, new Node(5), null);
    temp2 = new Node(4, new Node(2, new Node(1), new Node(5)), new Node(10));
    
    System.out.println("\n[15] Should print out:\ntrue");
    System.out.println(isBST(root0));

    System.out.println("\n[16] Should print out:\ntrue");
    System.out.println(isBST(root4));

    System.out.println("\n[17] Should print out:\nfalse");
    System.out.println(isBST(temp));

    System.out.println("\n[18] Should print out:\nfalse");
    System.out.println(isBST(temp2));

    System.out.println("\n[19] Should print out:\nfalse");
    System.out.println(isBST(temp2));
    
    System.out.println("\nTesting isDegenerate ....");
    
    System.out.println("\n[20] Should print out:\nfalse");
    System.out.println(isDegenerate(root0));

    System.out.println("\n[21] Should print out:\ntrue");
    System.out.println(isDegenerate(root2));

    System.out.println("\n[22] Should print out:\nfalse");
    System.out.println(isDegenerate(root3));

    System.out.println("\n[23] Should print out:\nfalse");
    System.out.println(isDegenerate(root4));

    System.out.println("\nTesting isPerfect ....");
    
    System.out.println("\n[24] Should print out:\ntrue");
    System.out.println(isPerfect(root0));
    
    System.out.println("\n[25] Should print out:\nfalse");
    System.out.println(isPerfect(root3));

    System.out.println("\n[26] Should print out:\ntrue");
    System.out.println(isPerfect(root1));

    System.out.println("\nTesting equals ....");

    System.out.println("\n[27] Should print out:\ntrue");
    System.out.println(equals(root0, null));

    System.out.println("\n[28] Should print out:\nfalse");
    System.out.println(equals(root1, temp));

    int[] A2 = {5, 2, 9, 1, 4, 12, 7};
    temp = null;
    temp = insertArray(A2, temp);

    System.out.println("\n[29] Should print out:\ntrue");
    System.out.println(equals(root1, temp));



    System.out.println("\nJust for fun....");

    boolean done = false;
    System.out.println("\n[30] Should print out:\nHigh Five!");
    if (done)
      System.out.println("High Five!");
    else
      System.out.println("Nope!");
*/

  }

}
